/* Copyright (c) 2007 Airsource Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#include "AEEAppGen.h" 
#include "AEEStdLib.h"
#include "testduma.bid"

#include "crt.hpp"
/*-------------------------------------------------------------------
Static function prototypes
-------------------------------------------------------------------*/



class MyApplet 
{
public:
	AEEApplet a;
	
public:
	// Deliberately empty constructor
	MyApplet(void) {}

	// No virtual methods allowed in this class, so non-virtual dtor
	~MyApplet(void) {}

	void initialise(void);

    static boolean HandleEvent(AEEApplet* pMe, AEEEvent eCode, uint16 wParam, uint32 dwParam);
	void DrawText(const AECHAR*);
	void ResetScreen(void);
	void DrawMenu(void);

private:
	int ybase;
};



#define MYAPPLET_FROM_APPLET(var) MyApplet* var = (MyApplet*)pMe;

int AEEClsCreateInstance(AEECLSID ClsId,
                         IShell * pIShell,
                         IModule * pMod,
                         void ** ppObj)
{
   *ppObj = NULL;
		
   if(AEEApplet_New( sizeof(MyApplet),                  // Size of our private class
                     ClsId,                              // Our class ID
                     pIShell,                            // Shell interface
                     pMod,                               // Module instance
                     (IApplet**)ppObj,                   // Return object
                     (AEEHANDLER)MyApplet::HandleEvent, // Our event handler
                     NULL))                              // No special "cleanup" function
      return(AEE_SUCCESS);

	return (EFAILED);
}


boolean MyApplet::HandleEvent(AEEApplet * pMe, 
                              AEEEvent eCode, 
                              uint16 wParam, 
                              uint32 dwParam)
{  
	MYAPPLET_FROM_APPLET(me);
    switch (eCode){
	case EVT_APP_START:                        
		me->DrawMenu();
        return(TRUE);

    case EVT_APP_STOP:
		memory_check();
        return(TRUE);


	case EVT_KEY:
		switch(wParam)
		{
		case AVK_1:
			// Leak
			new int;
			me->DrawText(L"Leaked 4 bytes");
			return TRUE;
		case AVK_2:
			{
				// Read off the end of a buffer
				char* buf = new char[5];
				DBGPRINTF("%c", buf[5]);
				delete[] buf;
				me->DrawText(L"Buffer overrun");
			}

			return TRUE;

		case AVK_3:
			{
				// Buffer underrun (write)
				char* buf = new char[5];
				buf[-1] = 0;
				// Buffer underruns are checked on deallocation.
				delete[] buf;
				me->DrawText(L"Buffer underrun");
			}
			return TRUE;

		default:
			{
				me->ResetScreen();
			}
			break;
		};
		break;

    default:
         break;
   }
   return(FALSE);
}

void MyApplet::ResetScreen(void)
{
	ybase = 0;
    IDISPLAY_ClearScreen(a.m_pIDisplay);
}

void MyApplet::DrawText(const AECHAR* text)
{
    IDISPLAY_DrawText(a.m_pIDisplay,    // Display instance
                      AEE_FONT_BOLD,       // Use BOLD font
                      text,              // Text - Normally comes from resource
                      -1,                  // -1 = Use full string length
                      0,                   // Ignored - IDF_ALIGN_CENTER
                      ybase,                   // Ignored - IDF_ALIGN_MIDDLE
                      NULL,                // No clipping
                      IDF_ALIGN_LEFT);
    IDISPLAY_Update (a.m_pIDisplay);
	ybase += 20;
}

void MyApplet::DrawMenu(void)
{
	ResetScreen();
	DrawText(L"TestDUMA");
	DrawText(L"(1) Generate a leak");
	DrawText(L"(2) Buffer overrun (read)");
	DrawText(L"(3) Buffer underrun (write)");
	DrawText(L"");
}


